import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import ddf.minim.spi.*; 
import ddf.minim.signals.*; 
import ddf.minim.*; 
import ddf.minim.analysis.*; 
import ddf.minim.ugens.*; 
import ddf.minim.effects.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class Journey_of_Fate_v4 extends PApplet {

// Journey of Fate re-factored
// by Dennis Carr AKA ihavefivehat
// 2013

// Import sound library:







// Objects for audio playback
Minim minim;
Sounder music;

// The game's font:
PFont font;

// Booleans for interaction
boolean click, did_once, state_change;

float chance_to_trip, trip;

// The current state of the game
int state;

// The current scene
int scene_number;

// The number of scenes in total
int number_of_scenes;

// I don't know what this is:
int text_state;

String[] fairyDialogue;

// For remembering the order that the actors should appear in.
ArrayList<Actor> sceneActors;

// The various actors
Player player;
Wizard wizard;
Monster monster;
Fairy fairy;
Villagers villagers;


Alarm mouseTimer;


public void setup() {

  size(800, 600);

  // Initialize the game's font:
  font = loadFont("TitleScreen.vlw");

  // Initialize minim and the Sounder object which plays the music
  minim = new Minim(this);
  music = new Sounder();

  mouseTimer = new Alarm(0);

  // Initialize non-actor objects
  player = new Player();
  
  chance_to_trip = 0.0f;
  trip = 1;

  // Assign each actor a certain race
  StringList possibleRaces = new StringList();
  possibleRaces.append("sandwich");
  possibleRaces.append("dodecahedron");
  possibleRaces.append("crab");
  possibleRaces.append("poop");
  possibleRaces.shuffle();
  String wizardRace = possibleRaces.get(0);
  String monsterRace = possibleRaces.get(1);
  String fairyRace = possibleRaces.get(2);
  String villagersRace = possibleRaces.get(3);

  // Assign each actor to a certain location:
  StringList possibleLocations = new StringList();
  possibleLocations.append("forest");
  possibleLocations.append("desert");
  possibleLocations.append("mountains");
  possibleLocations.append("caves");
  possibleLocations.shuffle();
  String wizardLocation = possibleLocations.get(0);
  String monsterLocation = possibleLocations.get(1);
  String fairyLocation = possibleLocations.get(2);
  String villagersLocation = possibleLocations.get(3);

  // Initialize actors:
  wizard = new Wizard(wizardRace, wizardLocation);
  monster = new Monster(monsterRace, monsterLocation);
  fairy = new Fairy(fairyRace, fairyLocation);
  villagers = new Villagers(villagersRace, villagersLocation);

  // Decide which actors will appear in which scenes:
  number_of_scenes = 20;
  sceneActors = new ArrayList<Actor>();

  for (int i = 0; i < number_of_scenes; i++) {    

    // Choose a random actor for this scene
    int actor_choice = PApplet.parseInt( random(4) );
    // Put that actor into sceneActors
    if      (actor_choice == 0) sceneActors.add(fairy);
    else if (actor_choice == 1) sceneActors.add(monster);
    else if (actor_choice == 2) sceneActors.add(villagers);
    else if (actor_choice == 3) sceneActors.add(wizard);
    
    // If the same actor is appearing in the last three scenes, do it over.
    if ( i > 1 && sceneActors.get(i-2) == sceneActors.get(i-1) && sceneActors.get(i-1) == sceneActors.get(i) ) {
      sceneActors.remove(i);
      i -= 1;
    } else {
      Actor currentActor = sceneActors.get(i);
      println("Scene " +i+ ": " +currentActor.race_name+ " " +currentActor.actor_name+
        " of the " +currentActor.biome_name);
    }
  }

  // Set various things:
  click = false;
  did_once = false;

  state = 0;
}


public void draw() {
  // Play the music
  music.play();

  // Run the mouse timer
  mouseTimer.run();

  if (state == 0) {
    displayTitle();
    if (click) state++;
  } else if (state == 1) {
    displayInstructionalText();
    if (click) state++;
  }


  // Actor scenes
  else if (state > 1 && state < 99) {
    scene_number = state-2;
    Actor currentActor;

    // Get the actor for this scene
    if (scene_number < number_of_scenes-1 && chance_to_trip < trip) {
      currentActor = sceneActors.get(scene_number);
//      monster.dead = true;
//      currentActor = villagers;
      currentActor.display();
      currentActor.action();
    }
    else if (scene_number >= number_of_scenes-1) {
      currentActor = sceneActors.get(scene_number-1);
      currentActor.display();
      textBox("You turn around to leave and suddenly trip. Your head smashes into a rock and splits open gruesomely. Oops! Guess that's just how things go.");
      player.dead = true;
    }
    else {
      currentActor = sceneActors.get(scene_number-1);
      currentActor.display();
      textBox("You turn around to leave and suddenly trip. Your head smashes into a rock and splits open gruesomely. Oops! Guess that's just how things go.");
      player.dead = true;
    }

    if (click) {      
      // If the player is dead, show the game over screen
      if (player.dead) {
        state = 99;
      }
      
      // If the player won the game, show the win screen
      if (player.won_game) {
        state = 100;
      }
            
      // 1: Check if any variables need to be changed. If so, change them.
      
      
      // If the player should gain health, do it.
      if (player.gain_health) {
        player.health += 1;
        player.gain_health = false;
      }
      
      // If the player should lose health, do it.
      if (player.apply_damage) {
        player.health -= currentActor.attack_power;
        player.apply_damage = false;
      }
      
      // If the current actor should loose health, do it
      if (currentActor.apply_damage) {
        currentActor.health -= player.attack_power;
        currentActor.apply_damage = false;
      }
      
      if (player.gain_special_hug) {
        player.special_hug_obtained = true;
        player.gain_special_hug = false;
      }

      // 2: Handle dialgoue
      
      currentActor.line_index++;
      
      // If the current dialogue is finished, load the next dialouge and reset the line
      // index.
      if (currentActor.dialogue_finished) {
        currentActor.currentDialogue = currentActor.nextDialogue;
        currentActor.line_index = 0;
        currentActor.dialogue_finished = false;
      }

      // If the scene is finished, reset the current actor's phrase_index and then proceed to
      // the next state.
      if (currentActor.scene_finished) {
        println("Scene " +scene_number+ " finished");
        trip = random(1);
        if (scene_number > 10) chance_to_trip += 0.05f;
        println(chance_to_trip);
        currentActor.reset();
        currentActor.scene_finished = false;
        state++;
        state = constrain(state, 0, number_of_scenes+1);
      }
    }
  }
  else if (state == 99) {
    gameOver();
  }
  else if (state == 100) {
    win();
  }

  click = false;
}


public void mouseClicked() {
  if (mouseTimer.finished) {
    click = true;
    mouseTimer = new Alarm(10);
  }
}
class Fairy extends Actor
{
  // A list to hold the various possible answers to the fairy's riddle:
  StringList riddleGuesses;

  int right_answer;
  
  Fairy(String race_name_, String biome_name_) {
    super(race_name_, biome_name_);
    actor_name = "fairy";

    health = 3;
    
    currentDialogue = new FairyIntro();

    // Create a list of answers for the fairy's riddle and then mix it up.
    riddleGuesses = new StringList();
    riddleGuesses.append("Bunnies!");
    riddleGuesses.append("The answer can only be time itself!");
    riddleGuesses.append("A lonesome cowboy?");
    riddleGuesses.append("Um... a river...?");
    riddleGuesses.append("I don't care about your dumb power anyway!");
    riddleGuesses.append("The moon?");
    riddleGuesses.append("Love? Sweet tender love?");
    riddleGuesses.shuffle();
  }

  public void display() {
    super.display();
    // If the fairy is not dead, display its image
    if (dead == false) shape(actorShape, width/2, 200);
  }

  public void action() {
    super.action();
  }
  
  public void reset() {
    currentDialogue = new FairyIntro();
  }
} // class end
class Monster extends Actor {
  
  // Whether the monster has been hugged:
  boolean hugged;
    
  Monster(String race_name_, String biome_name_) {
    super(race_name_, biome_name_);
    actor_name = "monster";
    
    currentDialogue = new MonsterIntro();
    
    hugged = false;
    
    health = 3;
  }
  
  public void display() {
    super.display();
    // If the monster is not dead and has not been hugged, display it's image
    if (!dead && !hugged) shape(actorShape, width/2, 200, 400, 400);
  }
  
  public void action() {
    super.action();
  }
  
  public void reset() {
    currentDialogue = new MonsterIntro();
  }
}
class Villagers extends Actor {
  // Is the village chief dead
  boolean chief_dead;
    
  Villagers(String race_name_, String biome_name_) {
    super(race_name_, biome_name_);
    actor_name = "villagers";
    
    currentDialogue = new VillagersIntro();
    
    chief_dead = false;
  }
  
  public void display() {
    super.display();
    // If all of the villagers are alive, display them all
    if (!dead && !chief_dead) {
      shape(actorShape, width*0.5f,       200, 150, 150);
      shape(actorShape, width*0.5f - 200, 200, 150, 150);
      shape(actorShape, width*0.5f + 200, 200, 150, 150);
    }
    // If the chief is dead, only display two villagers
    else if (!dead && chief_dead) {
      shape(actorShape, width*0.5f - 200, 200, 150, 150);
      shape(actorShape, width*0.5f + 200, 200, 150, 150);
    }
  }
  
  public void action() {
    super.action();
  }
  
  public void reset() {
    currentDialogue = new VillagersIntro();
  }
  
}
class Wizard extends Actor {
  
  int annoyance_level;
  boolean hugged;
  
  Wizard(String race_name_, String biome_name_) {
    super(race_name_, biome_name_);
    actor_name = "wizard";
    
    currentDialogue = new WizardIntro();
    
    annoyance_level = 0;
    health = 2;
    attack_power = 2;
    hugged = false;
  }
  
  public void display() {
    super.display();
    if (!dead) shape(actorShape, width/2, 200);
  }
  
  public void action() {
    super.action();
  }
  
  public void reset() {
    currentDialogue = new WizardIntro();
  }
}
class FairyIntro extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      if (lastActorName() == "fairy") textBox("You remain in the "+fairy.biome_name+" of the "+fairy.race_name+" "+fairy.actor_name+".");
      else textBox("You find yourself in the "+fairy.biome_name+" of the "+fairy.race_name+" "+fairy.actor_name+".");
      finished = true;
    }

    if (finished) {
      // If this is the first time meeting the fairy and she is alive
      if (!fairy.dead && fairy.first_encounter && !player.helm_obtained && !player.cookie_obtained) fairy.nextDialogue = new FirstEncounterGreeting();
      else if (!fairy.dead && !fairy.first_encounter && !player.special_hug_obtained && !player.helm_obtained && !player.cookie_obtained && !player.enslaved_villager_obtained) fairy.nextDialogue = new WelcomeBack();
      else if (!fairy.dead && player.special_hug_obtained && !player.cookie_obtained && !player.helm_obtained && !player.enslaved_villager_obtained) fairy.nextDialogue = new PlayerHugsFairy();
      else if (!fairy.dead && player.helm_obtained) fairy.nextDialogue = new PlayerFightsFairy();
      else if (fairy.dead) fairy.nextDialogue = new PlayerContemplatesMorality();
      else if (!fairy.dead && player.cookie_obtained && !player.net_obtained && !player.helm_obtained) fairy.nextDialogue = new FairyNoticesCookie();
      else if (!fairy.dead && player.enslaved_villager_obtained && !player.helm_obtained) fairy.nextDialogue = new PlayerGivesEnslavedVillager();
    }
  }
}

class PlayerGivesEnslavedVillager extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The fairy speaks:");
    if (line_index == 1) textBox("'Ah! You have done it! I can almost taste the endless stream of cookies pouring into my mouth! Give me the villager, quickly!");
    if (line_index == 2) textBox("You hand over the villager. The fairy opens the net and shakes the poor "+villagers.race_name+" out into a cage, then quickly shuts and locks the door.");
    if (line_index == 3) textBox("The cage contains a little oven and a fully stocked kitchen. The villager sighs resignedly and gets to work.");
    if (line_index == 4) textBox("The fairy speaks:");
    if (line_index == 5) textBox("'You truly are a great hero! Soon I shall control the production of cookies across all the land!'");
    if (line_index == 6) textBox("'They have always said that she who controls the flow of cookies controls the flow of history. Yes... a new era shall dawn. One in which the fairy is Queen!'");
    if (line_index == 7) textBox("'Here, take this treasure. You have earned it, and besides, I shall have no more need of it. I have everything I need right here.");
    if (line_index == 8) textBox("The fairy hands you the Legendary Treasure of Numinous Wonder. You have done what no other hero has dared to dream! You posess what has been called the most powerful item in the entire multiverse!");
    if (line_index == 9) textBox("You look down at the glowing treasure in your hands.");
    if (line_index == 10) textBox("As your eyes adjust to the magical glow, you begin to make out the treasure's wondrous form.");
    if (line_index == 11) textBox("It's a burrito!");
    if (line_index == 12) textBox("You take a bite.");
    if (line_index == 13) textBox("It's the most delicious burrito you've ever tasted! You take another bite and find yourself utterly immersed in a reverie of incredible flavor.");
    if (line_index == 14) {
      textBox("With every bite that you take, the other end of the burrito grows longer. It's an infinite burrito!");
      finished = true;
    }
    if (finished) {
      if (player.cursed == true) fairy.nextDialogue = new WizardCurseKillsPlayer();
      else fairy.nextDialogue = new PlayerWinsGame();
    }
  }
}


class PlayerWinsGame extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Your life is complete. There is honestly nothing you can imagine that could possibly top this current experience.");
    if (line_index == 1) {
      textBox("You retire into the country side, living a quiet life full of the never-ending enrichment that only a burrito can provide.");
      player.won_game = true;
      finished = true;
    }
    if (finished) player.won_game = true;
  }
}


class WizardCurseKillsPlayer extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Suddenly you hear a strange voice in your head. It sounds strangely familiar...");
    if (line_index == 1) textBox("Gradually it dawns on you... this is the voice of the "+wizard.race_name+" wizard! It's his death curse coming back to haunt you!");
    if (line_index == 2) textBox("A chunk of the burrito begins to squirm in your mouth, as if it has a life of it's own. It jumps down your trachea!");
    if (line_index == 3) textBox("As you slowly choke to death, the wizard's voice echoes in your head over and over again, mocking you with cruel laughter.");
    if (line_index == 4) {
      textBox("Oh well, better luck next time!");
      player.dead = true;
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class FairyNoticesCookie extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The fairy is about to speak, but suddenly stops and begins sniffing the air frantically.");
    if (line_index == 1) textBox("The fairy speaks:");
    if (line_index == 2) textBox("'Oh my! Is that what I think it is?! Is that one of the cookies baked by the "+villagers.race_name+" villagers of the "+villagers.biome_name+"?'");
    if (line_index == 3) textBox("'Please! I beg of you! I must have that cookie!'");
    if (line_index == 4) textBox("You shrug and hand over your Delicious Cookie to the fairy.");
    if (line_index == 5) textBox("The fairy grabs it and begins slurping it down in a matter that is, frankly, a little disturbing.");
    if (line_index == 6) textBox("The fairy speaks:");
    if (line_index == 7) textBox("'Thank you, kind sir! It has been too long since I tasted that exquisite flavor! You see... the other fairies have kept me from tasting them since... well, since last time.'");
    if (line_index == 8) textBox("'They the cookies make me act insane! Ha! What do they know!'");
    if (line_index == 9) textBox("'But don't worry about that! Listen, I try to keep this information secret, but I'm the keeper of the Legendary Treasure of Numinous Wonder. If you can secure me a reliable source of those cookies, it's yours no questions asked.'");
    if (line_index == 10) {
      textBox("'Here, take this net. Let's just say it's good for catching small, helpless creatures. You can do with it as you wish, but don't forget my offer. Here, have some healing magic for your trouble.'");
      player.cookie_obtained = false;
      player.net_obtained = true;
      finished = true;
    }
    if (finished) fairy.nextDialogue = new FairyHealsPlayer();
  }
}


class PlayerContemplatesMorality extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You stare at the dead fairy and contemplate the morality of your actions.");
    if (line_index == 1) {
      textBox("Or you would, that is, if you weren't consumed with the Helm's power.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "fairy") fairy.nextDialogue = new PlayerKicksObjectsAndStays();
      else fairy.nextDialogue = new PlayerKicksObjectsAndLeaves();
    }
  }
}


class PlayerKicksObjectsAndStays extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Instead, you hang around for a while randomly kicking objects around you.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class PlayerKicksObjectsAndLeaves extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Instead, you randomly kick a few objects and leave.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class PlayerFightsFairy extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("When you see the fairy standing in front of you, the Helm's effects take hold! You run forward brandishing your weapon!");
    if (line_index == 1) {
      textBox("The fairy gets ready to meet you in combat!");
      fairy.apply_damage = true;
      player.apply_damage = true;
    }
    if (line_index == 2) {
      textBox("After the ensuing battle, you have "+player.health+" health points. The fairy has "+fairy.health+" health points.");
      finished = true;
    }
    if (finished) {
      if (player.health <= 0) fairy.nextDialogue = new FairyKillsPlayer();
      else if (fairy.health <= 0) fairy.nextDialogue = new PlayerKillsFairy();
      else {
        if (nextActorName() == "fairy") fairy.nextDialogue = new PlayerContinuesToFightFairy();
        else fairy.nextDialogue = new PlayerRunsAwayFromFairy();
      }
    }
  }
}


class PlayerContinuesToFightFairy extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You decide to stay and continue fighting.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class PlayerRunsAwayFromFairy extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You run away.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class PlayerKillsFairy extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The fairy lies dead! You roar loudly in triumph.");
      fairy.dead = true;
    }
    if (line_index == 1) textBox("Suddenly you notice something shiny in the fairy's remains. You pick it up.");
    if (line_index == 2) textBox("It's the Legendary Treasure of Numinous Wonder, the most powerful object in the known universe!");
    if (line_index == 3) textBox("With this item in your posession, your journey would be at a triumphant end!");
    if (line_index == 4) {
      textBox("But under the influence of the Helm, you are unable to recognize it.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "fairy") fairy.nextDialogue = new PoundsChestLikeApe();
      else fairy.nextDialogue = new SmashTreasureAndLeave();
    }
  }
}


class PoundsChestLikeApe extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You throw it to the ground and smash it with your feet. You then stand around and pound your chest in an ape-like fashion.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class SmashTreasureAndLeave extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You smash it under your foot and leave.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}



class FairyKillsPlayer extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You have died!");
      finished = true;
    }
    if (finished) player.dead = true;
  }
}



class PlayerHugsFairy extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Before the fairy can say anything, you reach out and give it a loving hug.");
    if (line_index == 1) textBox("The fairy returns your embrace and speaks:");
    if (line_index == 2) {
      textBox("'Why thank you, hero! Now please allow me to return your kindness with my healing magic!'");
      finished = true;
    }
    if (finished) fairy.nextDialogue = new FairyHealsPlayer();
  }
}


class WelcomeBack extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("'Welcome back, brave wanderer! Do you need more healing?");
      finished = true;
    }
    if (finished) fairy.nextDialogue = new FairyHealsPlayer();
  }
}


class FirstEncounterGreeting extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("'Welcome, brave wanderer! I am the magical "+fairy.race_name+" fairy!'");
    if (line_index == 1) {
      textBox("'Please allow me to heal your wounds!'");
      fairy.first_encounter = false;
      finished = true;
    }

    if (finished) fairy.nextDialogue = new FairyHealsPlayer();
  }
}


class FairyHealsPlayer extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The fairy casts healing magic upon you, and you gain health.");
      player.gain_health = true;
    }
    if (line_index == 1) textBox("You now have " +player.health+ " health points.");
    if (line_index == 2) {
      textBox("What a kind fairy!");
      finished = true;
    }

    if (finished) {
      // If the player has not yet answered the riddle correctly
      if (player.special_hug_obtained == false) fairy.nextDialogue = new FairyAsksRiddle();
      // If not...
      else if (player.special_hug_obtained == true) {
        // If the player is going to stay with the fairy for the next scene..
        if (nextActorName() == "fairy") fairy.nextDialogue = new StickAround();
        else fairy.nextDialogue = new ReturnToWanderings();
      }
    }
  }
}


class FairyAsksRiddle extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Before you go, wanderer, let me ask you a riddle!");
    if (line_index == 1) {
      textBox("If you answer correctly, I shall bestow upon you a special power!");
      fairy.riddleGuesses.shuffle();
    }
    if (line_index == 2) {
      textBox("The fairy asks you the riddle. It's a tough one.");
      finished = true;
    }

    if (finished) fairy.nextDialogue = new PlayerAnswersRiddle();
  }
}


class PlayerAnswersRiddle extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("After some deep thought, you reply: '" +fairy.riddleGuesses.get(0)+ "'");
      fairy.right_answer = PApplet.parseInt( random(1, 7) );
      finished = true;
    }

    if (finished) {
      // If the player gave the right answer..
      if (fairy.right_answer == 2) fairy.nextDialogue = new PlayerAnswersCorrectly();
      else fairy.nextDialogue = new PlayerAnswersIncorrectly();
    }
  }
}


class PlayerAnswersCorrectly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The fairy looks impressed!");
    if (line_index == 1) textBox("'Congratulations, wise one! '"+fairy.riddleGuesses.get(0)+"' is the correct answer!'");
    if (line_index == 2) textBox("'I now grant you a most powerful ability!'");
    if (line_index == 3) textBox("You have gained the Special Hug power!");
    if (line_index == 4) textBox("You can now envelop unsuspecting passerby in a warm, motherly embrace!");
    if (line_index == 5) {
      textBox("It's sure to warm even the most wicked heart!");
      player.gain_special_hug = true;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "fairy") fairy.nextDialogue = new BaskInApproval();
      else fairy.nextDialogue = new ContinueTriumphantly();
    }
  }
}


class ContinueTriumphantly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Feeling like a pretty smart guy, you continue triumphantly on your adventures.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class BaskInApproval extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Wow, look at Mr. Smart Guy over here!");
    if (line_index == 1) textBox("You start to think that maybe Mother was right and you really are special.");
    if (line_index == 2) {
      textBox("You stick around a bit longer, wanting to bask in the fairy's approval.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class PlayerAnswersIncorrectly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The fairy shakes her head condescendingly.");
    if (line_index == 1) textBox("'Haha! Wait... you're serious? Uh... no... '" +fairy.riddleGuesses.get(0)+ " is not correct.'");
    if (line_index == 2) {
      textBox("'I guess you can try again next time... if you want.'");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "fairy") fairy.nextDialogue = new PlayerTriesAgain();
      else fairy.nextDialogue = new PlayerLeavesInShame();
    }
  }
}


class PlayerLeavesInShame extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You leave quickly, hoping to avoid further embarassment.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class PlayerTriesAgain extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Though embarassed, you decide to stick around. Maybe the fairy will at least heal you again.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class ReturnToWanderings extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Thanking the fairy, you return to your wanderings.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}


class StickAround extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You decide to stick around to see if the fairy will heal you again.");
      finished = true;
    }
    if (finished) fairy.scene_finished = true;
  }
}
class MonsterIntro extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      if (lastActorName() == "monster") textBox("You remain in the "+monster.biome_name+" of the "+monster.race_name+" "+monster.actor_name+".");
      else textBox("You find yourself in the "+monster.biome_name+" of the "+monster.race_name+" "+monster.actor_name+".");
      finished = true;
    }
    if (finished) {
      if (!monster.dead && !player.special_hug_obtained && !monster.hugged) monster.nextDialogue = new MonsterAttacks();
      else if (!monster.dead && !monster.hugged && player.special_hug_obtained) monster.nextDialogue = new PlayerHugsMonster();
      else if (monster.hugged) monster.nextDialogue = new PlayerScreamsIntoVoid();
      else if (monster.dead) monster.nextDialogue = new MonstersBodyLiesOnGround();
    }
  }
}


class MonstersBodyLiesOnGround extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The monster's body lies rotting on the ground.");
      finished = true;
    }
    if (finished) {
      if (player.special_hug_obtained == true) monster.nextDialogue = new PlayerHugsMonstersCorpse();
      else {
        if (nextActorName() == "monster") monster.nextDialogue = new PlayerAdmiresOwnHandiwork();
        else monster.nextDialogue = new PlayerReflectsOnOwnHeroism();
      }
    }
  }
}


class PlayerReflectsOnOwnHeroism extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("After spending some time reflecting on your incredible heroism and remarkable good looks, you decide to move on.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class PlayerAdmiresOwnHandiwork extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You decide to stay a bit longer to admire your handiwork.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}

class PlayerHugsMonstersCorpse extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You activate your Special Hug ability!");
    if (line_index == 1) {
      textBox("You bend down and wrap your arms around the corpse, covering it in a warm embrace!");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "monster") monster.nextDialogue = new PlayerWondersWhyHeDidThat();
      else monster.nextDialogue = new PlayerHopesNoOneSaw();
    }
  }
}


class PlayerHopesNoOneSaw extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Nothing happens. You leave quickly, hoping no one saw what you just did.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}



class PlayerWondersWhyHeDidThat extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Nothing happens. You stay and wonder why you did that.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class PlayerScreamsIntoVoid extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("There is no one here.");
    if (line_index == 1) {
      textBox("The emptiness fills you with rage! You scream unintelligibly into the void.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "monster") monster.nextDialogue = new PlayerScreamsForLongTime();
      else monster.nextDialogue = new PlayerIsSatisfiedAndLeaves();
    }
  }
}


class PlayerIsSatisfiedAndLeaves extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Satisfied, you turn around and leave.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class PlayerScreamsForLongTime extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You hang out and keep screaming for a weirdly long time.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class PlayerHugsMonster extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The "+monster.race_name+" monster seems ready to attack!");
    if (line_index == 1) textBox("But before it makes a move, you rush in and use your Special Hug!");
    if (line_index == 2) textBox("The monster seems confused at first, but then its demeanor softens. A single tear falls from its eye.");
    if (line_index == 3) textBox("The monster speaks:");
    if (line_index == 4) textBox("'Thank you, kind human. I know that I project an air of aggression, but that's really just my way of reaching out for help. You're the only one who has ever cared enough to love me!'");
    if (line_index == 5) textBox("'Please, take this as a token of my appreciation... I don't need it anymore.'");
    if (line_index == 6) textBox("The monster takes something off of its head and throws it to you! It then turns around and walks dramatically into the sunset.");
    if (line_index == 7) textBox("You look at what you now hold in your hand. It's the Helm of Monstrous Rage, a Legendary Artifact! You place it on your head.");
    if (line_index == 8) {
      textBox("Suddenly all your thoughts begin to be overtaken by a blind and murderous rage!");
      monster.hugged = true;
    }
    if (line_index == 9) {
      textBox("Your last thought before you are transformed into a violent and irrational psychopath is that perhaps this wasn't such a good idea.");
      player.helm_obtained = true;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "monster") monster.nextDialogue = new PlayerPunchesDirt();
      else monster.nextDialogue = new PlayerMakesOddGruntingNoises();
    }
  }
}


class PlayerPunchesDirt extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You notice the dirt below your feet. Something about it makes you incredibly angry!");
    if (line_index == 1) textBox("You squat and begin punching the ground, making angry growls each time you throw your fist.");
    if (line_index == 2) {
      textBox("You remain in this position for quite some time.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class PlayerMakesOddGruntingNoises extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You run off, making a series of odd grunting noises as you go.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class MonsterAttacks extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The " +monster.race_name+ " monster lunges in to attack!");
      player.apply_damage = true;
      monster.apply_damage = true;
    }
    if (line_index == 1) textBox("After the ensuing battle, you have "+player.health+" health points.");
    if (line_index == 2) {
      textBox("The monster has "+monster.health+" health points.");
      finished = true;
    }
    if (finished) {
      if (player.health <= 0) monster.nextDialogue = new MonsterKillsPlayer();
      else if (monster.health <= 0) monster.nextDialogue = new PlayerKillsMonster();
      else {
        if (nextActorName() == "monster") monster.nextDialogue = new PlayerContinuesToFight();
        else monster.nextDialogue = new PlayerRunsAway();
      }
    }
  }
}


class PlayerContinuesToFight extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You decide to stay and continue fighting.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class PlayerRunsAway extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You run away.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class MonsterKillsPlayer extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You have died!");
      finished = true;
    }
    if (finished) player.dead = true;
  }
}


class PlayerKillsMonster extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You've slain the monster!");
      monster.dead = true;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "monster") monster.nextDialogue = new PlayerEnjoysVictory();
      else monster.nextDialogue = new PlayerRunsOffIntoDistance();
    }
  }
}


class PlayerRunsOffIntoDistance extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You run off into the distance, excited to tell everyone about how great you are!");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}


class PlayerEnjoysVictory extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You decide to stay for a while and enjoy the feeling of victory.");
      finished = true;
    }
    if (finished) monster.scene_finished = true;
  }
}
class VillagersIntro extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      if (lastActorName() == "villagers") textBox("You remain in the "+villagers.biome_name+" of the "+villagers.race_name+" "+villagers.actor_name+".");
      else textBox("You find yourself in the "+villagers.biome_name+" of the "+villagers.race_name+" "+villagers.actor_name+".");
      finished = true;
    }
    if (finished) {
      if (!villagers.dead && player.helm_obtained) villagers.nextDialogue = new PlayerSlaughtersVillagers();
      else if (!villagers.dead && player.special_hug_obtained && !player.cookie_obtained && !player.helm_obtained && !player.net_obtained && !villagers.chief_dead) villagers.nextDialogue = new PlayerHugsVillageChief();
      else if (!villagers.dead && villagers.first_encounter && !monster.dead) villagers.nextDialogue = new VillagersFirstEncounterDialogue();
      else if (monster.dead && !villagers.dead && !player.helm_obtained && !player.net_obtained && !player.cookie_obtained) villagers.nextDialogue = new PlayerAnnouncesTriumph();
      else if (!villagers.dead && player.cookie_obtained && !player.net_obtained && !villagers.chief_dead) villagers.nextDialogue = new VillagersCannotOfferAnythingElse();
      else if (villagers.dead && villagers.chief_dead && monster.dead && !player.helm_obtained && !player.net_obtained) villagers.nextDialogue = new PlayerCautiouslyAnnouncesTriumph();
      else if (villagers.dead && !player.net_obtained) villagers.nextDialogue = new VillageIsDeserted();
      else if (!villagers.dead && lastActorName() == "villagers" && !player.helm_obtained && !player.special_hug_obtained && !monster.dead) villagers.nextDialogue = new VillagerSobsQuietly();
      else if (!villagers.dead && !villagers.chief_dead && !player.helm_obtained && !player.net_obtained && !player.special_hug_obtained && !villagers.first_encounter) villagers.nextDialogue = new VillageElderBugsPlayer();
      else if (!player.helm_obtained && !player.enslaved_villager_obtained && player.net_obtained) villagers.nextDialogue = new PlayerUsesNet();
      else villagers.nextDialogue = new VillageIsDeserted();
    }
  }
}


class PlayerUsesNet extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You enter the village.");
    if (line_index == 1) {
      if (villagers.dead || villagers.chief_dead) textBox("Everybody is still hiding from you. You hear a rustling sound in a nearby bush. Just then, you realize why the fairy gave you the net.");
      else {
        if (monster.dead) textBox("The villagers crowd around you in admiration. You can smell cookies baking in the distant houses. Suddenly it occurs to you why the fairy gave you the net.");
        else textBox("The villagers look at you mournfully. You can smell cookies baking in the distant houses. Suddenly you realize why the fairy gave you a net.");
      }
    }
    if (line_index == 2) {
      if (villagers.dead || villagers.chief_dead) textBox("You throw a stone into the bush to scare the villager out of hiding. As soon as it emerges, you toss out the fairy's net. It works! The villager is totally trapped!");
      else textBox("You take out the net. The villagers look at it confusedly. You feel a little bad as you toss out the net and ensnare the nearest "+villagers.race_name+". The rest of the villagers let out shocked yelps and run into their houses.");
    }
    if (line_index == 3) {
      if (nextActorName() == "villagers") textBox("Just for fun, you spend some time poking the ensnared villager.");
      else textBox("You grab up the enslaved villager and go off on your way.");
      player.enslaved_villager_obtained = true;
      villagers.dead = true;
      player.enslaved_villager_obtained = true;
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class VillageElderBugsPlayer extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Not these depressing villagers again! You were just starting to mellow out after the last time you ran into them.");
    if (line_index == 1) textBox("You try to casually walk away, but they have already noticed you.");
    if (line_index == 2) textBox("The village chief calls out to you:");
    if (line_index == 3) textBox("'Oh great warrior! Have you killed the monster yet? Please! You are our only hope!'");
    if (line_index == 4) {
      textBox("You're really getting sick of this guy's endless nagging. Can't he see that you're busy with important tasks?");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerTriesToChangeTheSubject();
      else villagers.nextDialogue = new PlayerMakesVagueExcuses();
    }
  }
}


class VillagerSobsQuietly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You stand around with the villagers hoping someone will break the ice. They all seem pretty upset by the whole monster situation and no one's talking.");
    if (line_index == 1) {
      textBox("One of them begins sobbing quietly. Things are getting awkward fast.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerDoesntFeelLikeWalking();
      else villagers.nextDialogue = new PlayerSlipsAwayQuietly();
    }
  }
}


class PlayerSlipsAwayQuietly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("On second thought, it's probably time to get out of here.");
    if (line_index == 1) {
      textBox("You wait until the villagers are more completely distracted by their sadness and slip away quietly.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerDoesntFeelLikeWalking extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("This is a pretty depressing situation. You would leave, but you just don't feel like walking anywhere right now. Maybe the villagers will start to cheer up if you wait around for a while.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerCautiouslyAnnouncesTriumph extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You enter the village and cautiously announce your triumph over the monster, not sure if it will be enough to smooth over the whole complication from earlier.");
    if (line_index == 1) {
      textBox("No one makes so much as a noise.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerTriesAnnouncingAgain();
      else villagers.nextDialogue = new PlayerLeavesQuickly();
    }
  }
}


class PlayerLeavesQuickly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You leave quickly.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerTriesAnnouncingAgain extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Perhaps the villagers just didn't hear you. You decide to try again.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class VillagersCannotOfferAnythingElse extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The village chief calls out to you:");
    if (line_index == 1) {
      textBox("'Welcome back, great hero! While we are overjoyed to see you in our village again, we cannot offer you anything but our thanks as we have already given you all that we can spare.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerEnjoysBeingAdored();
      else villagers.nextDialogue = new PlayerLeavesInAHuff();
    }
  }
}


class PlayerLeavesInAHuff extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("What a bunch of ungrateful little jerks! You leave in a huff.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerEnjoysBeingAdored extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("Being appreciated by these villagers is good for your self esteem. you decide to stay for a little while longer.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class VillageIsDeserted extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The village is deserted and eerily quiet.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerEnjoysSilence();
      else villagers.nextDialogue = new PlayerLeavesQuicklyUnsure();
    }
  }
}


class PlayerLeavesQuicklyUnsure extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You leave quickly, not sure why you came back in the first place.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerEnjoysSilence extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You stay for a while, enjoying the silence.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerAnnouncesTriumph extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("As you enter the "+villagers.biome_name+", you loudly announce your triumph over the monster which had been plaguing the villagers.");
    if (line_index == 1) textBox("You revel in their joyous shouts and tears of relief.");
    if (line_index == 2) textBox("The village chief calls out to you:");
    if (line_index == 3) textBox("'Oh thank you, great warrior! You truly are the hero of this age!'");
    if (line_index == 4) {
      textBox("'Though we don't have much to give you, please accept this Delicious Cookie as our payment!");
      player.cookie_obtained = true;
    }
    if (line_index == 5) {
      textBox("You have to admit that you were hoping for something more valuable, but you accept the cookie anyway.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerSoaksInVillagersLove();
      else villagers.nextDialogue = new PlayerTakesCookieAndLeaves();
    }
  }
}


class PlayerTakesCookieAndLeaves extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You tuck it into your pocket and leave.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerSoaksInVillagersLove extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("It's been so long since your deeds were validated by the approval of others. You decide to stay for a while to soak in the villagers love for you.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerHugsVillageChief extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("As soon as you see the miserable little villagers, you know exactly what to do to lift their spirits. Special Hug activate!");
    if (line_index == 1) textBox("You grab the village elder and squeeze him in the most loving way possible.");
    if (line_index == 2) textBox("Suddenly you hear a sickening snapping sound and he cries out in pain. It seems that his tiny body couldn't take the force of your hug!");
    if (line_index == 3) {
      textBox("You drop his now lifeless body and back away in horror.");
      villagers.chief_dead = true;
      finished = true;
    }
    if (finished) {
      if (player.cookie_obtained == true) villagers.nextDialogue = new PlayerUttersMeekly();
      else villagers.nextDialogue = new VillagerOffersCookieInTerror();
    }
  }
}


class PlayerUttersMeekly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The other villagers shriek in terror and scatter off in all directions.");
    if (line_index == 1) {
      if (nextActorName() == "villagers") textBox("You utter a meek apology into the now empty "+villagers.biome_name+".");
      else textBox("You utter a meek apology into the now deserted village and leave as quickly as possible.");
      villagers.dead = true;
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}



class VillagerOffersCookieInTerror extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The other villagers skriek in terror and scatter off in all directions.");
    if (line_index == 1) textBox("One of the villagers calls out to you:");
    if (line_index == 2) textBox("'Please, don't hurt us! We beg of you! Just take this Delicious Cookie and leave us in peace!'");
    if (line_index == 3) {
      textBox("She grovels in front of you and holds out the cookie in offering.");
      player.cookie_obtained = true;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerCantFindApology();
      else villagers.nextDialogue = new PlayerBacksAwaySlowly();
    }
  }
}


class PlayerBacksAwaySlowly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You take the cookie and then back away slowly, leaving the "+villagers.biome_name+" as casually as possible");
      villagers.dead = true;
      finished = true;
    }
    if (finished) {
      villagers.dead = true;
      villagers.scene_finished = true;
    }
  }
}


class PlayerCantFindApology extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You take the cookie and stand there awkwardly trying to come up with a fitting apology.");
      villagers.dead = true;
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerSlaughtersVillagers extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The helm kicks in as soon as you lay your eyes on the pitiful villagers.");
    if (line_index == 1) {
      textBox("You let out an angry roar and start slaughtering everyone you see.");
      villagers.dead = true;
    }
    if (line_index == 2) {
      textBox("It isn't long before the every villager within your eye sight is dead. You feel a vague sense of guilt which is quickly drowned out by the Helm's power.");
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerChewsOnSword();
      else villagers.nextDialogue = new PlayerLumbersOutOfVillage();
    }
  }
}


class PlayerLumbersOutOfVillage extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You lumber out of the "+villagers.biome_name+" mumbling angry-sounding jibberish.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerChewsOnSword extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You yell and then spend some time chewing visciously on your sword.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class VillagersFirstEncounterDialogue extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The village chief calls out to you:");
    if (line_index == 1) textBox("'O great warrior! Our people are in need of your skill!'");
    if (line_index == 2) textBox("'The "+monster.race_name+" monster of the "+monster.biome_name+" has been murdering our people. It killed my daughter!'");
    if (line_index == 3) {
      textBox("'We beg of you! Go and slay the monster and our people shall be forever in your debt!'");
      villagers.first_encounter = false;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "villagers") villagers.nextDialogue = new PlayerThinksVillagersAreCoolGuys();
      else villagers.nextDialogue = new VillagersAreHarshingPlayersMellow();
    }
  }
}


class PlayerMakesVagueExcuses extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You make some vague excuses and leave hurriedly.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class PlayerTriesToChangeTheSubject extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You keep talking and try to change the subject. It doesn't work.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}


class VillagersAreHarshingPlayersMellow extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("These villagers are really starting to harsh your mellow with their loud pleading. You wish they'd just take a chill pill and relax for a while.");
    if (line_index == 1) {
      textBox("You find an opportunity to slip out without making things too awkward.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}



class PlayerThinksVillagersAreCoolGuys extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("These villagers seem like cool guys. You decide to hang out with them for a while.");
      finished = true;
    }
    if (finished) villagers.scene_finished = true;
  }
}
class WizardIntro extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      if (lastActorName() == "wizard") textBox("You remain in the "+wizard.biome_name+" of the "+wizard.race_name+" "+wizard.actor_name+".");
      else textBox("You find yourself in the "+wizard.biome_name+" of the "+wizard.race_name+" "+wizard.actor_name+".");
      finished = true;
    }

    if (finished) {
      if (wizard.first_encounter && !player.cookie_obtained && !player.helm_obtained && !player.special_hug_obtained && wizard.annoyance_level == 0) wizard.nextDialogue = new WizardFirstEncounter();
      else if (!wizard.dead && !wizard.first_encounter && player.helm_obtained) wizard.nextDialogue = new WhosAnnoyingNow();
      else if (wizard.dead) wizard.nextDialogue = new FlyHoversInAir();
      else if (!wizard.dead && player.special_hug_obtained && !player.helm_obtained && !wizard.hugged) wizard.nextDialogue = new PlayerHugsWizard();
      else if (!wizard.dead && wizard.annoyance_level == 1 && !player.helm_obtained && !player.cookie_obtained && !wizard.hugged) wizard.nextDialogue = new PlayerAnnoysWizard();
      else if (!wizard.dead && wizard.annoyance_level == 2 && !player.helm_obtained && !player.cookie_obtained && !wizard.hugged && wizard.health == 1) wizard.nextDialogue = new WizardWantsToFightAgain();
      else if (!wizard.dead && wizard.annoyance_level == 2 && !player.helm_obtained && !player.cookie_obtained && !wizard.hugged) wizard.nextDialogue = new WizardWantsToFight();
      else if (!wizard.dead && wizard.first_encounter && player.helm_obtained) wizard.nextDialogue = new BloodCurdlingScream();
      else if (!wizard.dead && player.cookie_obtained && !player.helm_obtained) wizard.nextDialogue = new WizardTakesCookie();
      else if (!wizard.dead && wizard.hugged && !player.helm_obtained) wizard.nextDialogue = new WizardIsInLove();
    }
  }
}


class WizardTakesCookie extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("As soon as you enter, the wizard notices the Delicious Cookie that you carry in your inventory.");
    if (line_index == 1) textBox("The wizard speaks:");
    if (line_index == 2) {
      if (wizard.hugged) textBox("'Dearest one! Thou hast brought a token of our love! How didst thou know I had a taste for sweets?'");
      else textBox("'Ah! I have always had a taste for sweets! Give me that cookie and I shall reward you greatly!");
    }
    if (line_index == 3) textBox("Before you have time to reply, the wizard levitates a chunk of the cookie out of your inventory and into his hand.");
    if (line_index == 4) textBox("The wizard gulps down the cookie with a series of sickening slurps. At first he seems very satisfied, but then his eyes bulge and his skin turns pale.");
    if (line_index == 5) textBox("The wizard speaks:");
    if (line_index == 6) textBox("'Argh! Didst yon cookies contain hazelnuts?! I doth possess severe nut allgeries! Why was there not a warning label?!'");
    if (line_index == 7) {
      textBox("'Thou hast killed me! Foul knave! I shall curse you with the blackest of hexes!'");
      finished = true;
    }
    if (finished) wizard.nextDialogue = new WizardCursesPlayer();
  }
}



class WizardIsInLove extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The wizard speaks:");
    if (line_index == 1) {
      if (lastActorName() == "wizard") textBox("'Tis wondrous to spend time with you, dearest one.'");
      else textBox("'Ah, 'tis you! The embrace we shared hast never left my mind!'");
    }
    if (line_index == 2) {
      if (nextActorName() == "wizard") textBox("You stay and give the wizard a hug- more out of pity than anything else.");
      else textBox("You leave quickly, hoping the wizard can take a hint.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class PlayerHugsWizard extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Before the wizard can say a word, you activate your Special Hug power.");
    if (line_index == 1) textBox("You reach out and ensnare the wizard in an almost inhumanly warm embrace.");
    if (line_index == 2) textBox("You feel the wizard's demeanor soften almost instantly. You let him out of your arms and he brings his eyes up to gaze into yours.");
    if (line_index == 3) textBox("You start to pick up a weird vibe from him.");
    if (line_index == 4) textBox("The wizard speaks:");
    if (line_index == 5) textBox("'None before thou hast shown me such tenderness. Mine heart does swell with such emotion as I have never felt before. Wist thou take this gift as a token of mine love?");
    if (line_index == 6) {
      textBox("The wizard hands you a magical amulet. Your attacks will now deal 3 damage!");
      player.attack_power = 3;
      wizard.hugged = true;
    }
    if (line_index == 7) textBox("You thank the wizard and he gazes at you lovingly.");
    if (line_index == 8) {
      if (nextActorName() == "wizard") textBox("It's a bit creepy but you stay, hoping he'll provide you with more magical items.");
      else textBox("You hope he didn't get the wrong idea. You make an excuse and leave hastily.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class WizardWantsToFightAgain extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The wizard is still angry! You get ready for another fight.");
      finished = true;
    }
    if (finished) wizard.nextDialogue = new PlayerFightsWizard();
  }
}


class WizardWantsToFight extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The wizard speaks:");
    if (line_index == 1) textBox("'So be it. Mine patience is an end. Feel the wrath of my dark power!");
    if (line_index == 2) {
      textBox("Uh oh. You really did it this time! You ready yourself for the coming fight.");
      finished = true;
    }
    if (finished) wizard.nextDialogue = new PlayerFightsWizard();
  }
}


class PlayerAnnoysWizard extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      if (lastActorName() == "wizard") textBox("Art thou still here?");
      else textBox("Hast thou returned, annoying one?");
    }
    if (line_index == 1) textBox("'Thou art testing my patience! If it is power you seek, then take this magical trinket and begone!'");
    if (line_index == 2) textBox("'But returneth not, or I shall gladly supply the painful death which you so seem to desire.'");
    if (line_index == 3) {
      textBox("Wow! The wizard tossed you a magical sword! You will now do twice as much damage with each attack!");
      player.attack_power = 2;
      wizard.annoyance_level = 2;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "wizard") wizard.nextDialogue = new PlayerMakesWeirdNoisesInWizardsEar();
      else wizard.nextDialogue = new PlayerIsAnAnnoyingBastard();
    }
  }
}


class PlayerIsAnAnnoyingBastard extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("People always said you could be an annoying bastard, but you never thought being irritating could pay so well!");
    if (line_index == 1) {
      textBox("You run off clutching your new sword to your chest and giggling maniacally.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class PlayerMakesWeirdNoisesInWizardsEar extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Did the wizard really just give you a prize for being irritating?");
    if (line_index == 1) textBox("You think that maybe if you do some more dumb stuff he'll give you something else.");
    if (line_index == 2) {
      textBox("You sidle up next to him and start making weird noises in his ear.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class FlyHoversInAir extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The wizard's corpse lies crumpled up on the floor. It's starting to smell a little bit. A fly hovers in the air.");
      finished = true;
    }
    if (finished) {
      if (!player.helm_obtained) wizard.nextDialogue = new PlayerPokesCorpseWithStick();
      else wizard.nextDialogue = new PlayerSmashesFly();
    }
  }
}


class PlayerSmashesFly extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You roar and smash the fly between your fists.");
    if (line_index == 1) {
      if (nextActorName() == "wizard") textBox("You then start swatting wildly at all the other flies, as well as at anything else within arm's reach.");
      else textBox("Satisfied, you turn around and stomp away.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class PlayerPokesCorpseWithStick extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You poke the corpse with a stick and wait to see if anyting happens.");
    if (line_index == 1) {
      if (nextActorName() == "wizard") textBox("Nothing does, but keep waiting anyway.");
      else textBox("Nothing does. You decide to leave before your gag reflexes fully kick in.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class BloodCurdlingScream extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You see a "+wizard.race_name+" wizard standing in front of you. The helm's effects take hold!");
    if (line_index == 1) {
      textBox("You let out a blood-curdling scream and start flailing your arms wildly. The fight is on!");
      finished = true;
    }
    if (finished) wizard.nextDialogue = new PlayerFightsWizard();
  }
}



class WhosAnnoyingNow extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("It's the "+wizard.race_name+" wizard. The helm's effects take hold!");
    if (line_index == 1) {
      textBox("'Who's annoying now?!' you yell as you run forward brandishing your weapon.");
      player.apply_damage = true;
      wizard.apply_damage = true;
      finished = true;
    }
    if (finished) wizard.nextDialogue = new PlayerFightsWizard();
  }
}


class PlayerFightsWizard extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("The wizard fires plumes of magic your way. You retaliate with your sword!");
      player.apply_damage = true;
      wizard.apply_damage = true;
    }
    if (line_index == 1) textBox("After the ensuing fight you have "+player.health+" health points.");
    if (line_index == 2) {
      textBox("The wizard has "+wizard.health+" health points.");
      finished = true;
    }
    if (finished) {
      if (player.health <= 0) wizard.nextDialogue = new WizardKillsPlayer();
      else if (wizard.health <= 0) wizard.nextDialogue = new PlayerKillsWizard();
      else {
        if (nextActorName() == "wizard") wizard.scene_finished = true;
        else wizard.nextDialogue = new PlayerRunsAwayFromWizard();
      }
    }
  }
}


class PlayerRunsAwayFromWizard extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You run away.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class PlayerKillsWizard extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("You have dealt the wizard a mortal blow!");
    if (line_index == 1) textBox("The wizard falls to his knees, the life seeping out of him. But before he dies, he raises his eyes to meet yours and speaks:");
    if (line_index == 2) {
      textBox("'Fool! Didst thou think that thou could kill such a powerful being with no consequenses to thineself!");
      finished = true;
    }
    if (finished) wizard.nextDialogue = new WizardCursesPlayer();
  }
}


class WizardCursesPlayer extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The wizard reaches into his robes and pulls out a dagger carved with evil-looking desings.");
    if (line_index == 1) textBox("He brings the dagger to his wrist and cuts into it deeply, letting a torrent of blood spill onto an arcane seal carved into the ground.");
    if (line_index == 2) textBox("His eyes turn pitch black and he mutters several syllables in a forgotten tongue before finally dying.");
    if (line_index == 3) {
      textBox("You're sure it's nothing to worry about.");
      player.cursed = true;
      wizard.dead = true;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "wizard") wizard.nextDialogue = new PlayerMakesSureWizardIsDead();
      else wizard.nextDialogue = new PlayerStepsOverWizardsBody();
    }
  }
}


class PlayerStepsOverWizardsBody extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You step over the wizard's body and make for the door.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class PlayerMakesSureWizardIsDead extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("But just in case, you wait to make sure the old man is truly dead.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class WizardKillsPlayer extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("You have died!");
      finished = true;
    }
    if (finished) player.dead = true;
  }
}


class WizardFirstEncounter extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("The wizard speaks:");
    if (line_index == 1) textBox("'Who doth enter my "+wizard.biome_name+" and interrupt my ancient "+wizard.race_name+" magicks?'");
    if (line_index == 2) {
      textBox("'Leave, intruder, lest ye be the target of my next spell!'");
      wizard.first_encounter = false;
      wizard.annoyance_level = 1;
      finished = true;
    }
    if (finished) {
      if (nextActorName() == "wizard") wizard.nextDialogue = new MorbidSenseOfCuriosity();
      else wizard.nextDialogue = new PlayerLeavesWizardAloneForTimeBeing();
    }
  }
}


class PlayerLeavesWizardAloneForTimeBeing extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) textBox("Perhaps it would be best to leave the wizard alone for the time being.");
    if (line_index == 1) {
      textBox("You decide to head somewhere else.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}


class MorbidSenseOfCuriosity extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("He seems pretty angry, but your morbid sense of curiosity compels you to stay a bit longer.");
      finished = true;
    }
    if (finished) wizard.scene_finished = true;
  }
}
class ClassName extends Dialogue {
  public void display(int line_index_)
  {
    super.display(line_index_);
    if (line_index == 0) {
      textBox("text");
      finished = true;
    }
  
    if (finished) {
    }
  }
}
class Actor {
  PShape actorShape;
  PImage backgroundImage;

  // The name of the region where this actor lives, and his/her name & race
  String biome_name, actor_name, race_name;
  
  int health;
  int attack_power;

  Dialogue currentDialogue, nextDialogue;

  // A string list which holds the dialogue which is currently being progressed through
  int[] line_order;

  // Keeping track of the current dialogue
  int dialogue_index, line_index;

  boolean dialogue_finished, scene_finished;
  boolean first_encounter;
  boolean end_first_encounter;
  
  boolean apply_damage;

  boolean dead;

  Actor(String race_name_, String biome_name_) {
    actorShape = loadShape(race_name_+ ".svg");
    backgroundImage = loadImage(biome_name_+ ".jpg");
    
    health = 1;
    attack_power = 1;

    race_name = race_name_;
    biome_name = biome_name_;
    
    first_encounter = true;

    dialogue_index = 0;
    line_index = 0;
  }

  public void display() {
    shapeMode(CENTER);
    // Draw background
    image(backgroundImage, 0, 0);
    // Draw text box
    fill(0);
    rectMode(CORNER);
    rect(0, 400, width, height);
  }

  public void action() {
    
    if (currentDialogue != null) {
      currentDialogue.display(line_index);

      // Check if the current dialogue is finished
      if (currentDialogue.finished == true) {
        dialogue_finished = true;
      }
    }
  }
  
  public void reset() {
  }
}
class Alarm
{
  float targetMillis;
  boolean finished;

  Alarm(float targetMillis_)
  {
    targetMillis = targetMillis_ + millis();
  }

  public void run()
  {
    if (millis() >= targetMillis) finished = true;
    else finished = false;
  }
}
class Dialogue {
  int line_index;
  
  boolean finished;
    
  Dialogue() {
    line_index = 0;
    finished = false;
  }
  
  public void display(int line_index_) {
    line_index = line_index_;
  }
}
class Player {
  int attack_power;
  int health;
  
  boolean dead;
  boolean won_game;
  
  boolean helm_obtained, cookie_obtained, special_hug_obtained, net_obtained, cursed,
          enslaved_villager_obtained;
          
  // If true, the player will have a variable affected next mouse click        
  boolean gain_health, gain_special_hug, apply_damage;
          
  Player() {
    health = 3;
    attack_power = 1;
  }
  
}
class Sounder {
  AudioPlayer music;
  
  Sounder() {
    music = minim.loadFile("journeyOfFate.wav");
  }
  
  // The current clip is played
  public void play() {    
    // If the music is finished, rewind it and start playing it again
    if (!music.isPlaying()) {
      music.rewind();
      music.play();
    } else {
      music.play();
    }
  }
 
  public void close() {
    music.close();
  }
  
}
public void gameOver() {
  
  fill(255,0,0);
  background(128);
  
  textAlign(CENTER);
  textFont(font, 200);
  text("GAME OVER",width/2,200);
  textFont(font, 50);
  text("You stink! Restart the program to test",width/2,300);
  text("your fate once again!",width/2,350);
}

public void win(){
  
  fill (0,255,0);
  background(128);
  
  textAlign(CENTER);
  textFont(font,100);
  text("CONGRATULATIONS",width/2,200);
  textFont(font,50);
  text("You have completed Journey of Fate.",width/2,300);
  text("now go and experience the endless admiration",width/2,350);
  text("of all your peers, as well as incredible success",width/2,400);
  text("in all aspects of life. You truly are a great person.",width/2,450);
}


public String nextActorName() {
  if ( scene_number < number_of_scenes-1) {
    Actor nextActor = sceneActors.get(scene_number+1);
    return nextActor.actor_name;
  }
  else return "";
}


public String lastActorName() {
  if (scene_number > 0) {
    Actor nextActor = sceneActors.get(scene_number-1);
    return nextActor.actor_name;
  }
  else return "";
}
public void textBox(String inputText_) {
  String inputText = inputText_;
  fill(255);
  // Display the text:
  rectMode(CORNER);
  text(inputText, 30, 420, 740, 180);
  
  // If the text is too long to fit on the screen, display an error message
  float w = textWidth(inputText);
  if (w > 2960) {
    fill(255, 0, 0);
    text("ERROR! TEXT TOO LONG! CORRECT IT, BUDDY!", 30, 400);
  }
}
public void displayTitle() {
  background(128);
  textFont(font, 50);
  textAlign(CENTER);
  fill(255);
  text("Journey", width/2, 200);
  text("of", width/2, 250);
  textFont(font, 100);
  text("FATE", width/2, 320);
  textFont(font, 40);
  text("Click to begin your journey... of FATE", width/2, 480);
}

public void displayInstructionalText() {
  background(128);
  textFont(font, 40);
  textAlign(LEFT);
  
  text("Welcome to JOURNEY OF FATE.", 30, 50);
  text("You are a lone wanderer in a land of mystery.", 30, 100);
  text("In this adventure, you have no control over your future.", 30, 150);
  text("Your only option is to CLICK THE MOUSE to see what", 30, 200);
  text("fate has in store for you.", 30, 250);
  text("Now go, great hero, go and claim your DESTINY!", 30, 350);
}
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "Journey_of_Fate_v4" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
